@@ -3,9 +3,6 @@ from sm.misc.deser import *
 from sm.misc.funcs import *
 from sm.misc.exp_manager import *
 from sm.misc.logger import *
-from sm.misc.parallel import Parallel, ParallelFnWrapper
-
-import sm.misc.graph as G
 
 
 class UnreachableError(Exception):
